package tfg;

import org.jsfml.audio.Sound;
import org.jsfml.audio.SoundBuffer;
import org.jsfml.audio.SoundSource;
import org.jsfml.graphics.Sprite;
import org.jsfml.graphics.Texture;

import java.io.IOException;
import java.nio.file.Paths;

/**
 * Holds everything that pertains to the player (you?).
 * @author Ritchie Cunningham
 */
public class Player extends Entity {
    /**
     * The texture for the sprite.
     */
    private final Texture entitySpritesheetTexture = new Texture();

    /**
     * The action the player is currently performing.
     */
    private PlayerAction currentAction = PlayerAction.NONE;
    /**
     * The buffer that allows the 'stuck' sound to be loaded.
     */
    private final SoundBuffer cannotMoveBuffer = new SoundBuffer();
    /**
     * The object that plays the 'stuck' sound.
     */
    private final Sound cannotMove = new Sound();
    
    /**
     * Create a new player with a location at (x,y).
     * @param x Starting x pos.
     * @param y Starting y pos.
     */
    public Player(int x, int y) {
        entityLoc = new Location(x, y);
        /* Load sprite texture and 'stuck' sound. */
        try {
            entitySpritesheetTexture.loadFromFile(Paths.get("res/player.png"));
            cannotMoveBuffer.loadFromFile(Paths.get("res/stuck.wav"));
        } catch(IOException ex) {
            ex.printStackTrace();
        }
        Sprite sprite = new Sprite(); /* Create a new regular sprite. */
        sprite.setTexture(entitySpritesheetTexture);
        /* Create a new animated sprite from the regular sprite. */
        entitySprite = new AnimatedSprite(sprite, entityLoc);
        cannotMove.setBuffer(cannotMoveBuffer);
    }

    /**
     * Create a new player at(0,0).
     */
    public Player() {
        this(0,0);
    }

    /**
     * Move the player in the specified direction.
     * @param d Direction to move the player.
     */
    public void move(Direction d) {
        if(currentAction == PlayerAction.NONE && entitySprite.finishedAnimating()) {
            /*
             * Get the location relative to the current location.
             * e.g. NORTH would return the location above the current location.
             */
            Location newLoc = entityLoc.getRelativeLocation(d);
            entityLoc.setDirection(d);
            if(parentMap.isValidLocation(newLoc)) {
                currentAction = PlayerAction.MOVING;
                entitySprite.startAnimation(AnimationType.WALKING);
            } else if(cannotMove.getStatus() == SoundSource.Status.STOPPED) {
                /*
                 * Play the stationary walk animation.
                 * It appears like the player is trying to move, but can't.
                 */
                entitySprite.startAnimation(AnimationType.STATIONARY_WALK);
                /* Play an annoying sound. */
                cannotMove.play();
            }
        }
    }


    /**
     * Update the animation progress.
     */
    public void update() {
        if(currentAction == PlayerAction.MOVING) {
            if(entitySprite.finishedAnimating()) {
                currentAction = PlayerAction.NONE;
                /* Actually move the location. */
                entityLoc = entityLoc.getRelativeLocation(entityLoc.getDirection());
                /* Update the sprite with new location. */
                entitySprite.updatePosition(entityLoc);            
            }
        }
        /* Update the animation, if there is no current animation, ignore. */
        entitySprite.animate();
    }
    
    /**
     * Get animated sprite.
     * @return Animated sprite.
     */
    public AnimatedSprite getAnimatedSprite() {
        return entitySprite;
    }

    public void interact() {
        Location interactLoc = entityLoc.getRelativeLocation(entityLoc.getDirection());
        Entity e = parentMap.getEntityatLocation(interactLoc);
        if(e != null && e instanceof NPC) {
            System.out.println("'Sup Nerd!");
            NPC npc = (NPC)e;
            npc.move(Direction.SOUTH);
        }
    }
}
